﻿using System;
using System.IO;
using System.Net.Sockets;

namespace VitesseClientDemo
{
    class AsyncClient
    {
        public string Address { get; private set; }
        public int Port { get; private set; }

        public Action<String> Logger { get; set; }

        private TcpClient Client { get; set; } = null;
        private StreamReader Reader { get; set; } = null;
        private StreamWriter Writer { get; set; } = null;

        public AsyncClient(string address, int port)
        {
            Address = address;
            Port = port;
        }

        public async void Start()
        {
            if (Client != null)
            {
                Dispose();
            }

            Client = new TcpClient();
            try
            {
                Client.Connect(Address, Port);
                NetworkStream nStream = Client.GetStream();
                Reader = new StreamReader(nStream);
                Writer = new StreamWriter(nStream);
                Writer.AutoFlush = true;
                OnServerConnected();

                while (true)
                {
                    string rxBuf = await Reader.ReadLineAsync();
                    if (rxBuf != null)
                    {
                        OnMessageReceived(rxBuf);
                    }
                    else
                    {
                        break;  // Client closed connection
                    }
                }
            }
            catch (Exception e)
            {
                if (Logger != null)
                {
                    Logger("Start: " + e.Message);
                }
            }

            Dispose();
        }

        public void Dispose()
        {
            if (Client != null)
            {
                OnServerDisconnected();
                Client.Close();
                Client = null;
                Reader = null;
                Writer = null;
            }
        }

        public void Send(string txBuf)
        {
            Writer.WriteLine(txBuf);
        }

        #region Events
        public delegate void ServerConnectedHandler(AsyncClient sender);

        public event ServerConnectedHandler ServerConnected;

        private void OnServerConnected()
        {
            if (ServerConnected != null)
            {
                ServerConnected(this);
            }
        }

        public delegate void ServerDisconnectedHandler(AsyncClient sender);

        public event ServerDisconnectedHandler ServerDisconnected;

        private void OnServerDisconnected()
        {
            if (ServerDisconnected != null)
            {
                ServerDisconnected(this);
            }
        }

        public delegate void MessageReceivedHandler(AsyncClient sender, string message);

        public event MessageReceivedHandler MessageReceived;

        private void OnMessageReceived(string message)
        {
            if (MessageReceived != null)
            {
                MessageReceived(this, message);
            }
        }

        #endregion
    }
}
